import type { ArxInputChangeEvent } from '@components/General/Input';
import { LitElement, css, html } from 'lit';
import { customElement, property, state } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';

import '@components/General/Input';

@customElement('arx-prompt')
export class EvePrompt extends LitElement {
  @property({ type: String }) promptText = 'Please provide input';
  @property({ type: String }) cancelText = 'Cancel';
  @property({ type: String }) saveText = 'Save';
  @property({ type: Boolean }) open = false;
  @property({ type: Boolean }) showInput = false;
  @property({ type: String }) placeholder = 'Enter your response';
  @property({ type: String }) defaultValue = '';

  @state() private _inputValue = '';
  private _previousFocus: HTMLElement | null = null;

  static override styles = css`
    :host {
      display: block;
    }

    .overlay {
      position: fixed;
      top: 0;
      left: 0;
      width: 100%;
      height: 100%;
      background-color: oklch(from var(--color-base-content) l c h / 0.6);
      display: flex;
      align-items: center;
      justify-content: center;
      z-index: 9999;
      opacity: 0;
      pointer-events: none;
      transition: opacity 0.2s ease;
      backdrop-filter: blur(4px);
    }

    .overlay.active {
      opacity: 1;
      pointer-events: all;
    }

    .prompt-container {
      background-color: var(--color-base-100);
      border-radius: var(--radius-box);
      border: var(--border) solid var(--color-base-300);
      box-shadow: calc(var(--depth) * 4px) calc(var(--depth) * 4px)
          calc(var(--depth) * 8px)
          oklch(from var(--color-base-content) l c h / 0.2),
        calc(var(--depth) * -1px) calc(var(--depth) * -1px)
          calc(var(--depth) * 4px) oklch(from var(--color-base-100) l c h / 0.4);
      width: 90%;
      max-width: 420px;
      padding: 28px;
      transform: scale(0.95) translateY(10px);
      transition: transform 0.25s cubic-bezier(0.1, 1, 0.2, 1);
      color: var(--color-base-content);
    }

    .overlay.active .prompt-container {
      transform: scale(1) translateY(0);
    }

    .prompt-header {
      font-size: 18px;
      font-weight: 600;
      margin: 0 0 16px 0;
      line-height: 1.4;
      color: var(--color-base-content);
    }

    .buttons {
      display: flex;
      justify-content: flex-end;
      gap: 12px;
      margin-top: 24px;
    }

    .prompt-container:focus-within {
      box-shadow: calc(var(--depth) * 4px) calc(var(--depth) * 4px)
          calc(var(--depth) * 8px)
          oklch(from var(--color-base-content) l c h / 0.2),
        calc(var(--depth) * -1px) calc(var(--depth) * -1px)
          calc(var(--depth) * 4px) oklch(from var(--color-base-100) l c h / 0.4),
        0 0 0 2px oklch(from var(--color-accent) l c h / 0.2);
    }
  `;

  constructor() {
    super();
    this._handleKeyDown = this._handleKeyDown.bind(this);
  }

  override connectedCallback() {
    super.connectedCallback();
    document.addEventListener('keydown', this._handleKeyDown);
  }

  override disconnectedCallback() {
    super.disconnectedCallback();
    document.removeEventListener('keydown', this._handleKeyDown);
  }

  override updated(changedProps: Map<string, unknown>) {
    if (changedProps.has('open') && this.open) {
      this._inputValue = this.defaultValue;
      this._previousFocus = document.activeElement as HTMLElement;

      // Focus the input or save button after rendering
      setTimeout(() => {
        if (this.showInput) {
          const input = this.shadowRoot?.querySelector('.input-field') as HTMLElement;
          if (input) input.focus();
        } else {
          const saveBtn = this.shadowRoot?.querySelector('.save-btn') as HTMLElement;
          if (saveBtn) saveBtn.focus();
        }
      }, 50);
    } else if (changedProps.has('open') && !this.open && this._previousFocus) {
      this._previousFocus.focus();
    }
  }

  private _handleKeyDown(e: KeyboardEvent) {
    if (!this.open) return;
    if (e.key === 'Escape') this._handleCancel();
    if (e.key === 'Enter' && !e.shiftKey) this._handleSave();
  }

  private _handleInputChange(e: ArxInputChangeEvent) {
    this._inputValue = e.detail.value;
  }

  private _handleCancel() {
    this.open = false;
    this.dispatchEvent(new CustomEvent('cancel'));
  }

  private _handleSave() {
    this.open = false;
    this.dispatchEvent(
      new CustomEvent('save', {
        detail: { value: this._inputValue },
      }),
    );
  }

  override render() {
    return html`
      <div
        class="${classMap({ overlay: true, active: this.open })}"
        @click="${(e: MouseEvent) => e.target === e.currentTarget && this._handleCancel()}"
      >
        <div class="prompt-container">
          <div class="prompt-header">${this.promptText}</div>

          ${
            this.showInput
              ? html`
                <arx-input
                  type="text"
                  class="input-field"
                  .value=${this._inputValue}
                  @change=${this._handleInputChange}
                  placeholder=${this.placeholder}
                ></arx-input>
              `
              : ''
          }

          <div class="buttons">
            <arx-button
              variant="accent"
              label=${this.cancelText}
              @click=${this._handleCancel}
            >
            </arx-button>
            <arx-button
              variant="secondary"
              label=${this.saveText}
              @click=${this._handleSave}
            >
            </arx-button>
          </div>
        </div>
      </div>
    `;
  }

  show() {
    this.open = true;
  }

  hide() {
    this.open = false;
  }

  getValue(): string {
    return this._inputValue;
  }
}
