import { getSigner, ndk } from '@/ndk';
import type { ArxInputChangeEvent } from '@components/General/Input';
import { NDKEvent } from '@nostr-dev-kit/ndk';
import { LitElement, css, html } from 'lit';
import { customElement, property, state } from 'lit/decorators.js';

import '@components/General/Button';
import '@components/General/Input';
import '@components/General/Textarea';

@customElement('arx-arbor-topic-creator')
export class ArborTopicCreator extends LitElement {
  @property({ type: String })
  categoryId = '';

  @state()
  private newTopic = '';

  @state()
  private topicContent = '';

  @state()
  private isCreating = false;

  static override styles = css`
    :host {
      display: block;
    }

    .error {
      color: red;
      margin-bottom: 1rem;
    }

    .category-id {
      color: #666;
      margin-bottom: 1rem;
      font-family: monospace;
    }
  `;

  private async doCreateTopic() {
    if (this.isCreating) return;

    if (this.newTopic.length < 3) {
      alert('Topic title must be at least 3 characters long');
      return;
    }

    if (this.topicContent.length < 10) {
      alert('Topic content must be at least 10 characters long');
      return;
    }

    this.isCreating = true;

    try {
      await getSigner();
      const event = new NDKEvent(ndk);
      event.kind = 892;
      event.tags = [
        ['name', this.newTopic],
        ['d', `60891:${this.categoryId}`],
      ];
      event.content = this.topicContent;
      await event.sign();
      await event.publish();

      this.dispatchEvent(
        new CustomEvent('topic-created', {
          bubbles: true,
          composed: true,
          detail: {
            topicId: event.id,
            categoryId: this.categoryId,
          },
        }),
      );

      this.newTopic = '';
      this.topicContent = '';
    } catch (error) {
      console.error('Failed to create topic:', error);
      alert('Failed to create topic');
    } finally {
      this.isCreating = false;
    }
  }

  private handleTopicInput(e: ArxInputChangeEvent) {
    this.newTopic = e.detail.value;
  }

  private handleContentInput(e: ArxInputChangeEvent) {
    this.topicContent = e.detail.value;
  }

  override render() {
    return html`
      <div class="category-id">Category ID: ${this.categoryId}</div>

      <arx-input
        type="text"
        placeholder="New Topic"
        .value=${this.newTopic}
        @change=${this.handleTopicInput}
        ?disabled=${this.isCreating}
      ></arx-input>

      <arx-textarea
        placeholder="Topic. You can use Markdown here."
        .value=${this.topicContent}
        @change=${this.handleContentInput}
        ?disabled=${this.isCreating}
      ></arx-textarea>

      <div class="button-group">
        <arx-button
          label="Cancel"
          @click=${() => this.dispatchEvent(new CustomEvent('cancel'))}
          ?disabled=${this.isCreating}
        ></arx-button>

        <arx-button
          label=${this.isCreating ? 'Creating...' : 'Create'}
          @click=${this.doCreateTopic}
          ?disabled=${this.isCreating}
        >
        </arx-button>
      </div>
    `;
  }
}
